// Atmel Studio 6
// Encoder und Tastenentprellung Atxmega128a1
// Encoder und Taster siehe unten 
// #define F_CPU 32000000UL //eingetragen in "Toolchain->AVR/GNU C Compiler->Symbols"
// USART 11520 Baud
// 32 Mhz intern

#include "tools.h"


#define PHASE_A     PORTB.IN & PIN4_bm
#define PHASE_B     PORTB.IN & PIN5_bm

volatile int8_t enc_delta;          // -128 ... 127
static int8_t last;

static FILE mystdout_uart = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);



int main(void)
{
	system_clocks_init();			// Clock	ein
	
	int8_t val = 0;
	
	PORTD.DIR  = 0xff;  		// PORTD0 Led Encoder
	PORTD.OUT  = 0x00;
	PORTC.DIRSET  = PIN0_bm;  		// PORTC0 Led fr KEY0 (Encoder-Key)
	PORTC.OUTSET  = PIN0_bm;

			
	PORTB.DIRCLR  = PIN0_bm | PIN1_bm | PIN2_bm | PIN3_bm | PIN4_bm | PIN5_bm; // Taster1 bis 4 und Encoder (Eingang_1 und Eingang_2)
	
	PORTB_PIN0CTRL = PORT_OPC_PULLUP_gc; // Pullup einschalten Taster Key0 - Key3
	PORTB_PIN1CTRL = PORT_OPC_PULLUP_gc;
	PORTB_PIN2CTRL = PORT_OPC_PULLUP_gc; 
	PORTB_PIN3CTRL = PORT_OPC_PULLUP_gc;
	PORTB_PIN4CTRL = PORT_OPC_PULLUP_gc; // Pullup einschalten Encoder Pin 4/5
	PORTB_PIN5CTRL = PORT_OPC_PULLUP_gc;

	TCC0.PER=0x7CFF; //  1ms fr Encoder bei 32Mhz 
	TCC0.CTRLA  |=  TC_CLKSEL_DIV1_gc;
	TCC0.INTCTRLA  |= TC_OVFINTLVL_HI_gc;
	
	TCC1.PER=0x9C3F; //  10ms fr Taster bei 32Mhz
	TCC1.CTRLA  |=  TC_CLKSEL_DIV1_gc;
	TCC1.INTCTRLA  |= TC_OVFINTLVL_MED_gc;

	
	PMIC.CTRL |= PMIC_HILVLEN_bm;
	PMIC.CTRL |= PMIC_MEDLVLEN_bm;
		
	sei();
	
	
	Rs232_Init();	// USARTE0 Init
	delay_ms(100);
								
	stdout = &mystdout_uart;	// bindet printf() ein, UART
	printf("\rn\rn UART ok\r\n");


	encode_init();


	while(1)
	{
			
			
			val += encode_read4();          // read a single step encoder
			PORTD.OUT  = val;				// PORTD0 LED an/aus 
			
			//delay_ms(200);
			
			if( get_key_press(1<<KEY0))		// Taster KEY0 toggle
				{
					PORTC.OUTTGL  = PIN0_bm;
					printf("%d",val);   //USART Ausgabe Encoder
					printf("   -> Encoder nach links oder rechts drehen und Taster druecken!\r\n");		
				}

	}
}

// System Clocks initialization intern 32 Mhz
void system_clocks_init(void)
{
	unsigned char n,s;
	s=SREG;
	cli();
	OSC.CTRL|=OSC_RC32MEN_bm;
	n=(CLK.PSCTRL & (~(CLK_PSADIV_gm | CLK_PSBCDIV1_bm | CLK_PSBCDIV0_bm))) |
	CLK_PSADIV_1_gc | CLK_PSBCDIV_1_1_gc; // Hier kann der Takt umgestellt werden!!
	CCP=CCP_IOREG_gc;
	CLK.PSCTRL=n;
	DFLLRC32M.CTRL&= ~DFLL_ENABLE_bm;
	while ((OSC.STATUS & OSC_RC32MRDY_bm)==0);
	n=(CLK.CTRL & (~CLK_SCLKSEL_gm)) | CLK_SCLKSEL_RC32M_gc;
	CCP=CCP_IOREG_gc;
	CLK.CTRL=n;
	OSC.CTRL&= ~(OSC_RC2MEN_bm | OSC_RC32KEN_bm | OSC_XOSCEN_bm | OSC_PLLEN_bm);
	PORTCFG.CLKEVOUT |= PORTCFG_CLKOUT_OFF_gc;
	SREG=s;
}

ISR(TCC0_OVF_vect)	// ENCODER
{
	 
	 int8_t new, diff;
	 new = 0;
	 if( PHASE_A )
	 new = 3;
	 if( PHASE_B )
	 new ^= 1;						// convert gray to binary
	 diff = last - new;				// difference last - new
	 if( diff & 1 ){				// bit 0 = value (1)
	 last = new;					// store new as next last
	 enc_delta += (diff & 2) - 1;	// bit 1 = direction (+/-)

	 }	
}


ISR(TCC1_OVF_vect)	// Tastenentprellung
{
	
	static uint8_t ct0, ct1, rpt;
	uint8_t i;
	i = key_state ^ ~KEY_PIN;		// key changed ?
	ct0 = ~( ct0 & i );			// reset or count ct0
	ct1 = ct0 ^ (ct1 & i);		// reset or count ct1
	i &= ct0 & ct1;			// count until roll over ?
	key_state ^= i;			// then toggle debounced state
	key_press |= key_state & i;		// 0->1: key press detect
	if( (key_state & REPEAT_MASK) == 0 )	// check repeat function
	rpt = REPEAT_START;		// start delay
	if( --rpt == 0 )
	{
		rpt = REPEAT_NEXT;			// repeat delay
		key_rpt |= key_state & REPEAT_MASK;
	}
	
}

int uart_putchar(char c, FILE *stream) 	// bindet printf() UART
{
	Rs232_SendByte(c);
	return 0;
}



void encode_init( void ) // Init Encoder
{
	int8_t new;
	
	new = 0;
	if( PHASE_A )
	new = 3;
	if( PHASE_B )
	new ^= 1;                   // convert gray to binary
	last = new;                   // power on state
	enc_delta = 0;
	
}	

int8_t encode_read1( void )         // read single step encoders
{
	int8_t val;
	
	cli();
	val = enc_delta;
	enc_delta = 0;
	sei();
	return val;                   // counts since last call
}


int8_t encode_read2( void )         // read two step encoders
{
	int8_t val;
	
	cli();
	val = enc_delta;
	enc_delta = val & 1;
	sei();
	return val >> 1;
}


int8_t encode_read4( void )         // read four step encoders
{
	int8_t val;
	
	cli();
	val = enc_delta;
	enc_delta = val & 3;
	sei();
	return val >> 2;
}



